# The qdrouter management schema

The schema `qdrouterd.json` is a JSON format file that defines annotations and
entity types of the Qpid Dispatch Router management model.  The model is based
on the AMQP management specification.

The schema is a JSON map with the following keys:

- "description": documentation string for the schema
- "prefix": Prefix prepended to schema names when they are exposed to AMQP management clients.
- "annotations": map of annotation names to definitions (see below)
- "entityTypes": map of entity type names to definitions (see below)

Annotation and entity type definition maps have the following keys:

- "description": documentation string.
- "operations": list of allowed operation names.
- "attributes": map of attribute names to attribute definitions (see below)

Entity type definitions also have these fields:

- "extends": Name of base type. The new type includes operations and attributes from the base type.
- "annotations": List of annotation names. The new type includes operations and attributes from all the annotations.

Attribute definition maps have the following fields:

- "type": one of the following:
  - "String": a unicode string value.
  - "Integer": an integer value.
  - "Boolean": a true/false value.
  - [...]: A list of strings is an enumeration. Values must be one of the strings or an integer integer index into the list, starting from 0.
- "default": a default can be a literal value or a reference to another attribute in the form `$attributeName`.

There is the following hierarchy among entity types:

`entity`: The base of all entity types.
- `configurationEntity`: base for all types that hold *configuration information*.

  Configuration information is supplied in advance and expresse *intent*. For
  example "I want the router to listen on port N". All the entities that can be
  used in the configuration file extend `configurationEntity`.

- `operationalEntity`: base for all types that hold *operational information*.

  Operational information reflects the actual current state of the router.  For
  example, "how many addresses are presently active?" All the entities queried
  by the `qdstat` tool extend `operationalEntity`.

The two types are often related. For example `listener` and `connector` extend
`configurationEntity`, they express the intent to make or receive
connections. `connection` extends `operationalEntity`, it holds information
the actual connection status.

The rest of this section provides the schema documentation in readable format.

## Annotations

### `addrPort`

Attributes for internet address and port.

Used by `listener`, `connector`.

`addr` (String, default=0.0.0.0)
:   Host address: ipv4 or ipv6 literal or a host name.

`port` (String, default=amqp)
:   Port number or symbolic service name.

### `saslMechanisms`

Attribute for a list of SASL mechanisms.

Used by `listener`, `connector`.

`saslMechanisms` (String, required)
:   Comma separated list of accepted SASL authentication mechanisms.

### `connectionRole`

Attribute for the role of a connection.

Used by `listener`, `connector`.

`role` (One of [normal, inter-router, on-demand], default=normal)
:   The role of an established connection. In the normal role, the connection is assumed to be used for AMQP clients that are doing normal message delivery over the connection.  In the inter-router role, the connection is assumed to be to another router in the network.  Inter-router discovery and routing protocols can only be used over interRouter connections.

### `sslProfile`

Attributes for setting TLS/SSL configuration for connections.

Used by `listener`, `connector`.

`certDb` (String)
:   The path to the database that contains the public certificates of trusted certificate authorities (CAs).

`certFile` (String)
:   The path to the file containing the PEM-formatted public certificate to be used on the local end of any connections using this profile.

`keyFile` (String)
:   The path to the file containing the PEM-formatted private key for the above certificate.

`passwordFile` (String)
:   If the above private key is password protected, this is the path to a file containing the password that unlocks the certificate key.

`password` (String)
:   An alternative to storing the password in a file referenced by passwordFile is to supply the password right here in the configuration file.  This option can be used by supplying the password in the 'password' option.  Don't use both password and passwordFile in the same profile.

## Base Entity Type

### `entity`

Base entity type for all entities.

`name` (String, unique)
:   Unique name, can be changed.

`identity` (String, unique)
:   Unique identity, will not change.

`type` (String, required)
:   Management entity type.

## Configuration Entities

### `configurationEntity`

Base type for entities containing configuration information.

Extends: `entity` (`name`, `identity`, `type`).

### `container`

Attributes related to the AMQP container.

Extends: `configurationEntity` (`name`, `identity`, `type`).

`containerName` (String)
:   The  name of the AMQP container.  If not specified, the container name will be set to a value of the container's choosing.  The automatically assigned container name is not guaranteed to be persistent across restarts of the container.

`workerThreads` (Integer, default=1)
:   The number of threads that will be created to process message traffic and other application work (timers, non-amqp file descriptors, etc.) .

`debugDump` (String)
:   A file to dump debugging information that can't be logged normally.

### `router`

Tracks peer routers and computes routes to destinations.

Extends: `configurationEntity` (`name`, `identity`, `type`).

`routerId` (String)
:   Router's unique identity.

`mode` (One of [standalone, interior, edge, endpoint], default=standalone)
:   In standalone mode, the router operates as a single component.  It does not participate in the routing protocol and therefore will not coorperate with other routers. In interior mode, the router operates in cooreration with other interior routers in an interconnected network.  In edge mode, the router operates with an uplink into an interior router network. Edge routers are typically used as connection concentrators or as security firewalls for access into the interior network.

`area` (String)
:   Unused placeholder.

`helloInterval` (Integer, default=1)
:   Interval in seconds between HELLO messages sent to neighbor routers.

`helloMaxAge` (Integer, default=3)
:   Time in seconds after which a neighbor is declared lost if no HELLO is received.

`raInterval` (Integer, default=30)
:   Interval in seconds between Router-Advertisements sent to all routers.

`remoteLsMaxAge` (Integer, default=60)
:   Time in seconds after which link state is declared stale if no RA is received.

`mobileAddrMaxAge` (Integer, default=60)
:   Time in seconds after which mobile addresses are declared stale if no RA is received.

`addrCount` (Integer)
:   Number of addresses known to the router.

`linkCount` (Integer)
:   Number of links attached to the router node.

`nodeCount` (Integer)
:   Number of known peer router nodes.

### `listener`

Listens for incoming connections to the router.

Extends: `configurationEntity` (`name`, `identity`, `type`).

Annotations: `sslProfile` (`certDb`, `certFile`, `keyFile`, `passwordFile`, `password`), `addrPort` (`addr`, `port`), `saslMechanisms` (`saslMechanisms`), `connectionRole` (`role`).

`requirePeerAuth` (Boolean, default=True)
:   Only for listeners using SSL.  If set to 'yes', attached clients will be required to supply a certificate.  If the certificate is not traceable to a CA in the ssl profile's cert-db, authentication fails for the connection.

`trustedCerts` (String)
:   This optional setting can be used to reduce the set of available CAs for client authentication.  If used, this setting must provide a path to a PEM file that contains the trusted certificates.

`allowUnsecured` (Boolean)
:   For listeners using SSL only.  If set to 'yes', this option causes the listener to watch the initial network traffic to determine if the client is using SSL or is running in-the-clear.  The listener will enable SSL only if the client uis using SSL.

`allowNoSasl` (Boolean)
:   If set to 'yes', this option causes the listener to allow clients to connect even if they skip the SASL authentication protocol.

`maxFrameSize` (Integer, default=65536)
:   Defaults to 65536.  If specified, it is the maximum frame size in octets that will be used in the connection-open negotiation with a connected peer.  The frame size is the largest contiguous set of uniterruptible data that can be sent for a message delivery over the connection. Interleaving of messages on different links is done at frame granularity.

### `connector`

Establishes an outgoing connections from the router.

Extends: `configurationEntity` (`name`, `identity`, `type`).

Annotations: `sslProfile` (`certDb`, `certFile`, `keyFile`, `passwordFile`, `password`), `addrPort` (`addr`, `port`), `saslMechanisms` (`saslMechanisms`), `connectionRole` (`role`).

`allowRedirect` (Boolean, default=True)
:   Allow the peer to redirect this connection to another address.

`maxFrameSize` (Integer, default=65536)
:   Maximum frame size in octets that will be used in the connection-open negotiation with a connected peer.  The frame size is the largest contiguous set of uniterruptible data that can be sent for a message delivery over the connection. Interleaving of messages on different links is done at frame granularity.

### `log`

Configure logging for a particular module.

Extends: `configurationEntity` (`name`, `identity`, `type`).

`module` (One of [ROUTER, MESSAGE, SERVER, AGENT, CONTAINER, CONFIG, ERROR, DISPATCH, DEFAULT], required)
:   Module to configure. The special module 'DEFAULT' specifies defaults for all modules.

`enable` (String, required, default=default)
:   Levels are: trace, debug, info, notice, warning, error, critical. The enable string is a comma-separated list of levels. A level may have a trailing '+' to enable that level and above. For example 'trace,debug,warning+' means enable trace, debug, warning, error and critical. The value 'none' means disable logging for the module. The value 'default' means use the value from the DEFAULT module.

`timestamp` (Boolean)
:   Include timestamp in log messages.

`source` (Boolean)
:   Include source file and line number in log messages.

`output` (String)
:   Where to send log messages. Can be 'stderr', 'syslog' or a file name.

### `fixedAddress`

Establishes semantics for addresses starting with a prefix.

Extends: `configurationEntity` (`name`, `identity`, `type`).

`prefix` (String, required)
:   The address prefix (always starting with '/').

`phase` (Integer)
:   The phase of a multi-hop address passing through one or more waypoints.

`fanout` (One of [multiple, single], default=multiple)
:   One of 'multiple' or 'single'.  Multiple fanout is a non-competing pattern.  If there are multiple consumers using the same address, each consumer will receive its own copy of every message sent to the address.  Single fanout is a competing pattern where each message is sent to only one consumer.

`bias` (One of [closest, spread], default=closest)
:   Only if fanout is single.  One of 'closest' or 'spread'.  Closest bias means that messages to an address will always be delivered to the closest (lowest cost) subscribed consumer. Spread bias will distribute the messages across subscribers in an approximately even manner.

### `waypoint`

A remote node that messages for an address pass through.

Extends: `configurationEntity` (`name`, `identity`, `type`).

`address` (String, required)
:   The AMQP address of the waypoint.

`connector` (String, required)
:   The name of the on-demand connector used to reach the waypoint's container.

`inPhase` (Integer, default=-1)
:   The phase of the address as it is routed _to_ the waypoint.

`outPhase` (Integer, default=-1)
:   The phase of the address as it is routed _from_ the waypoint.

## Operational Entities

### `operationalEntity`

Base type for entities containing current operational information.

Extends: `entity` (`name`, `identity`, `type`).

### `router.link`

Link to another AMQP endpoint: router node, client or other AMQP process.

Extends: `operationalEntity` (`name`, `identity`, `type`).

`linkName` (String)

`linkType` (One of [endpoint, waypoint, inter-router, inter-area])

`linkDir` (One of [in, out])

`owningAddr` (String)

`eventFifoDepth` (Integer)

`msgFifoDepth` (Integer)

`remoteContainer` (String)

### `router.address`

AMQP address managed by the router.

Extends: `operationalEntity` (`name`, `identity`, `type`).

`inProcess` (Boolean)

`subscriberCount` (Integer)

`remoteCount` (Integer)

`deliveriesIngress` (Integer)

`deliveriesEgress` (Integer)

`deliveriesTransit` (Integer)

`deliveriesToContainer` (Integer)

`deliveriesFromContainer` (Integer)

### `router.node`

AMQP node managed by the router.

Extends: `operationalEntity` (`name`, `identity`, `type`).

`addr` (String)

`nextHop` (Integer)

`routerLink` (Integer)

`validOrigins` (List)

### `connection`

Connections to the router's container.

Extends: `operationalEntity` (`name`, `identity`, `type`).

`container` (String)

`state` (One of [connecting, opening, operational, failed, user])

`host` (String)

`dir` (One of [in, out])

`role` (String)

`sasl` (String)

### `allocator`

Memory allocation pool.

Extends: `operationalEntity` (`name`, `identity`, `type`).

`typeSize` (Integer)

`transferBatchSize` (Integer)

`localFreeListMax` (Integer)

`globalFreeListMax` (Integer)

`totalAllocFromHeap` (Integer)

`totalFreeToHeap` (Integer)

`heldByThreads` (Integer)

`batchesRebalancedToThreads` (Integer)

`batchesRebalancedToGlobal` (Integer)
